/**
  Copyright (c) 2005-2007 by Turku PET Centre

  File: libtpcimgpv.c  

  Purpose: functions for printing the information on the library. 
  
  Version:

  2005-02-22 1.0.0 Jarkko Johansson

  2005-04-26 1.1.0 JJ
  Functions libtpcimgp_print_readme(), libtpcimgp_print_history(),
  libtpcimgp_print_dreadme() and libtpcimgp_print_dhistory()
  are used for getting the Readme and History messages for the libtpcimgp,
  with and without Doxygen style tags. Remember to update the
  functions to get correct Readme and History messages!

  Function libtpcimgp_print_build(), is used for getting the build
  information.
 
  2005-06-09 1.2.0 Kaisa sederholm
  files median.c, lms.c, lts.c ans mestim.c removed and added to
  libtpcmodel library

  2006-06-19 1.2.1 Kaisa sederholm
  Functions imgMatchMatrix and imgMatchHeader were added to imgproc.c.

  2006-08-08 1.3.0 KS
  Added imgfilter.c which includes functions for Gaussian filtering.

  2006-10-30 1.3.1 VO
  Added imgthrs.c and imgframe.

  2007-02-02 1.3.2 VO
  Removed IMG min and max search functions from this library.
  File imgproc.c contents splitted into several .c files. 

  2007-02-11       VO
  Added a few double backslashes for doxygen.
  Corrected the LGPL web address.

*/
/*****************************************************************************/

/** Library name. */
#define LIB_NAME "libtpcimgp"

/** Library version. */
#define LIB_VERSION "1.3.2"

/** Copyright. */
#define COPYRIGHT "(c) 2005-2007 by Turku PET Centre"

/*****************************************************************************/

#include <stdio.h>
#include "libtpcimgpv.h"

/*****************************************************************************/

/** 
    Print the library readme to specified FILE pointer, use stdout for screen.
*/
void libtpcimgp_print_readme(FILE *fp){

  fputs(
       "\n"
       LIB_NAME" "LIB_VERSION" "COPYRIGHT"\n"
       "\n"
       "Build "__DATE__" "__TIME__"\n"
       "\n"
       "\n"
       "PET Image Processing library   \n"
       "-------------------------------------------------------------------------------\n"
       "\n"
       "Libtpcimgp is distributed under the GNU Lesser General Public Licence\n"
       "http://www.turkupetcentre.net/software/lgpl-license.html\n"
       "More information on open source licences: http://www.opensource.org\n"
       "\n"
       "Description\n"
       "-------------------------------------------------------------------------------\n"
       "\n"
       "The libtpcimgp library is a collection of commonly used C files\n"
       "at Turku PET Centre for image processing.\n"
       "\n"
       "List of files\n"
       "-------------------------------------------------------------------------------\n"
       "\n"
       "Doxyfile        - Doxygen configuration file.\n"
       "\n"
       "include/        - Includes all the header files for the .c files\n"
       "\n"
       "Makefile        - The makefile\n"
       "\n"
       "** .c files **\n"
       "\n"
       "imgarithm.c     - Functions for image arithmetics.\n"
       "\n"
       "imgfilter.c     - Functions for doing Gaussian filtering.\n"
       "\n"
       "imgflip.c       - Functions for turning image volume.\n"
       "\n"
       "imgframe.c      - Functions for setting IMG frame times.\n"
       "\n"
       "imgmatch.c      - Functions for comparing IMG contents.\n"
       "\n"
       "imgscanner.c    - Functions for setting scanner specific parameters.\n"
       "\n"
       "imgthrs.c       - Functions for thresholding and outlier filtering.\n"
       "\n"
       "imgtransform.c  - Functions for image transformations.\n"
       "\n"
       "libtpcimgp.c    - Functions for running tests on libtpcimgp.\n"
       "\n"
       "libtpcimgpv.c   - Functions for printing information on this library.\n"
       "\n"
       "point.c         - Functions for processing with points.\n"
       "\n"
       "\n"
       "\n"
       "Using the library\n"
       "-------------------------------------------------------------------------------\n"
       "\n"
       "The program that uses the library should include the .h file it needs.\n"
       "Example 1:\n"
       "    \\#include <imgproc.h>  // includes all library functions\n"
       "Example 2:\n"
       "    \\#include <imgtransform.h>   // includes functions for transforms \n"
       "    \\#include <point.h>  // includes the 3D point structure\n"
       "    \\#include <libtpcimgpv.h>  // includes the information\n"
       "    \\#include <imgfilter.h>  // includes functions for Gaussian filtering\n"
       "\n"
       "libtpcimgp depends on libtpcimgio and libtpcmisc.\n"
       "Example:\n"
       "	gcc myprog.c -ltpcimgp -ltpcimgio -ltpcmisc \n"
       "\n",
       fp
       );

  fflush(fp);
}

/** 
    Print the library History to specified FILE pointer, 
    use stdout for screen.
*/
void libtpcimgp_print_history(FILE *fp)
{
  fputs(
	" \n"
	"2005-01-04 version 1.0.0 Vesa Oikonen, Calle Laakkonen, Riku Kln, Jarkko Johansson \n"
	"  Former libpet, the c	@ cd .. ommon PET C library, has been divided up in smaller	\n"
	"  sublibraries that each handle a specific task. This library includes functions \n"
	"  utilised in image processing.\n"
	"\n"
	"2005-04-21 1.0.1 JJ\n"
	"  Changes in files libtpcimgpv.c and .h:\n"
	"\n"
	"  Functions libtpcimgp_print_readme(), libtpcimgp_print_history(),\n"
	"  libtpcimgp_print_dreadme() and libtpcimgp_print_dhistory()\n"
	"  are used for getting the Readme and History messages for the libtpcimgp,\n"
	"  with and without Doxygen style tags. Remember to update the\n"
	"  functions to get correct Readme and History messages!\n"
	"\n"
	"  Function libtpcimgp_print_build(), is used for getting the build\n"
	"  information.\n"
	"\n"
	"2005-04-26 1.1.0 JJ\n"
	"  Compiled with MinGW.\n"
	"  Changes made in class point:\n"
	"  round() -> pRound() (conflict with math.h round())\n"
	"  PI = value -> \\#ifndef PI = value \\#endif\n"
	"\n"
	"2005-06-02 1.1.1 Kaisa Sederholm\n"
	"  mestim.c added \n"
        "\n"
        "2005-06-09 1.2.0 KS\n"
        "files median.c, lms.c, lts.c ans mestim.c removed and added to\n"
        "libtpcmodel library\n"
	"\n"
        "2006-06-19 1.2.1 KS\n"
        "Functions imgMatchMatrix and imgMatchHeader were added to imgproc.c.\n"
	"\n"
        "2006-08-08 1.3.0 KS\n"
        "imgfilter.c containing functions for Gaussian filtering was added.\n"
	"\n"
        "2006-10-30 1.3.1 VO\n"
        "imgthrs.c was added.\n"
	"\n"
        "2007-02-02 1.3.2 VO\n"
        "Max and min search functions moved into libtpcimgio.\n"
        "Added more functions for comparing IMG contents.\n"
	"\n",
	fp
	);

  fflush(fp);
}

/** 
    Print the library Readme to specified FILE pointer with Doxygen tags, 
    use stdout for screen.
*/
void libtpcimgp_print_dreadme(FILE *fp)
{
  fprintf(fp,"/**\n\\mainpage %s %s %s\n\\section readme Readme\n<PRE>\n",LIB_NAME,LIB_VERSION,COPYRIGHT);
  libtpcimgp_print_readme(fp);
  fprintf(fp,"\n</PRE> \n */\n");
}

/** 
    Print the library History to specified FILE pointer with Doxygen tags, 
    use stdout for screen.
*/
void libtpcimgp_print_dhistory(FILE *fp)
{
  fprintf(fp,"/**\n\\page History\n\\section history History\n<PRE>\n");
  libtpcimgp_print_history(fp);
  fprintf(fp,"\n</PRE> \n */\n");
}

/** 
    Print the build info to specified FILE pointer, use stdout for screen.
*/
void libtpcimgp_print_build(FILE *fp)
{
  fprintf(fp,"\n %s %s %s\n",LIB_NAME,LIB_VERSION,COPYRIGHT);
  fprintf(fp,"\n Build %s %s\n",__DATE__,__TIME__);
}


/*****************************************************************************/

